/***************************************************************************
	file:			tcn75a.c
	product:	API-005
	descr:		Diese API ist mit stack2Learn I2C-003 2-Wire Temperatur Sensor TCN75A kompatibel.
						
						Mehr Infos findes Du in tcn75a.h und auf stack2Learn.de
	
	date: 		11.2013 - first release

	version 	1.0
	compiler:	Keil Cx51 V9.05

	mc:				AT89C5131A
	crystal:	24MHz

	author: 	Viktor Schabelski info@approbe.de
	licence:	Creative Commons Lizenz
                http://www.stack2Learn.de/cc.html

**************************************************/

/*-----------------------------------------------------------------------------
	includes
---------------------------------------------------*/
#include <stdlib.h>
#include "tcn75a.h"
#include "tcn75a_config.h"
#include "../api-002_emuI2C/emuI2C.h"



/*-----------------------------------------------------------------------------
	prototype private functions
---------------------------------------------------*/



/*-----------------------------------------------------------------------------
	global variables
---------------------------------------------------*/
// Pointer: Ambient Temperature Register (TA)
#define TCN75A_TEMP_REGISTER		0x00
// Pointer: Sensor Configuration Register (CONFIG)
#define TCN75A_CONFIG_REGISTER	0x01


/*-----------------------------------------------------------------------------
	api
---------------------------------------------------*/
/*=====================================================================*//**
*	@brief		Diese Funktion initialisiert den TCN75A.
*						Ueber I2C wird die TCN75A_CONFIG_BYTE (tcn75a_config.h) Konstante
*						in den "Configuration" Register von TCN75A geschrieben.
*
*	@param		TCN75A_CONFIG_BYTE - 				Config Byte aus tcn75A_config.h
*	@param		TCN75A_SLAVE_ADDRESS -			Slave Adresse aus tcn75A_config.h
*	@return		TCN75A_SUCCESS
*	@return		TCN75A_WRITE_FAIL
************************************************************************/
errorCodeTCN75A_t TCN75A_Init (void){
	errorCodeTCN75A_t error = TCN75A_BUSY;
	unsigned char dataToSend[3] = {0x00};

	// Daten vorbereiten
	dataToSend[0] = TCN75A_SLAVE_ADDRESS;
	dataToSend[1] = TCN75A_CONFIG_REGISTER;
	dataToSend[2] = TCN75A_CONFIG_BYTE;

	// Daten senden
	error = EmuI2C_SendData(dataToSend, 3);
	if (error == EMU_I2C_SUCCESS){
		return (TCN75A_SUCCESS);
	}else{
		return (TCN75A_WRITE_FAIL);
	}
}


/*=====================================================================*//**
*	@brief		Diese Funktion liest die Umgebungstemperatur aus dem "Temperature"
*						Register via I2C aus.
*						An diese Funktion wird der Pointer auf eine float Variable uebergeben.
*						Der Temperaturwert wird in diese Variable gespeichert.
*
* @param		*temperature - Pointer auf float Variable fuer den gemessenen Temperaturwert
*	@param		TCN75A_SLAVE_ADDRESS -	Slave Adresse aus tcn75A_config.h
*	@return		TCN75A_SUCCESS
************************************************************************/
errorCodeTCN75A_t TCN75A_GetTemp (float* temperature){
	errorCodeTCN75A_t error = TCN75A_BUSY;
	unsigned char dataToSend[2] = {0x00};
	unsigned char dataToReceive[2] = {0x00};
	float multiplier = 0.0625f;
	
	// Daten vorbereiten
	dataToSend[0] = TCN75A_SLAVE_ADDRESS;
	dataToSend[1] = TCN75A_TEMP_REGISTER;

	// Daten senden
	error = EmuI2C_SendData(dataToSend, 2);
	if (error == EMU_I2C_SUCCESS){
		error = TCN75A_SUCCESS;
	}else{
		return (TCN75A_WRITE_FAIL);
	}

	// Daten vorbereiten
	dataToReceive[0] = TCN75A_SLAVE_ADDRESS + 1;
	
	// Daten lesen
	error = EmuI2C_ReceiveData(dataToReceive, 2);
	if (error == EMU_I2C_SUCCESS){
		error = TCN75A_SUCCESS;
	}else{
		return (TCN75A_READ_FAIL);
	}
	
	// negative TempWerte
	if(dataToReceive[0] >= 0x80){
		dataToReceive[0] = (~dataToReceive[0]) +1;
		multiplier *= (-1);
	}
	
	*temperature = (float)((dataToReceive[0] << 4) | (dataToReceive[1] >> 4));
 	*temperature *= multiplier;
	
	return (error);
}